#include "Preferences.h"
#include "XmlUtil.h"
#include <stdexcept>
#include <QErrorMessage>
#include <QXmlStreamReader>
#include <QXmlStreamWriter>

using namespace std;
using namespace XmlUtil;

namespace {

    /**
     * Convert the value <c>value</c> to a string and return the
     * result.
     *
     * @param[in] value value
     * @return value converted to a string
     */
    template<typename T> QString
    toString(const T& value)
    {
        std::ostringstream ostrm;
        ostrm << value;
        return QString::fromLatin1(ostrm.str().c_str());
    }

    /**
     * Convert the value <c>value</c> from a string to type
     * <c>T</c> and return the result.
     *
     * @param[in] value
     * @return value converted to a type <c>T</c>
     */
    template<typename T> T
    stringTo(const std::string& value)
    {
        T rv;
        std::istringstream istrm(value);
        istrm >> rv;
        return rv;
    }

}

namespace Harbinger {

    Preferences::Preferences()
        : advanceNotice(0),
          screensaver(0)
    {
        /* empty */
    }

    void
    Preferences::serialize(QIODevice& ostrm) const
    {
        QXmlStreamWriter xml(&ostrm);
        xml.setAutoFormatting(true);
        xml.writeStartDocument();
        xml.writeStartElement(QString::fromLatin1("harbinger"));
        xml.writeTextElement(QString::fromLatin1("advanceNotice"),
                             toString(this->advanceNotice));
        xml.writeTextElement(QString::fromLatin1("screensaver"),
                             toString(this->screensaver));
        xml.writeTextElement(QString::fromLatin1("message"),
                             toString(this->message));
        xml.writeEndElement();
        xml.writeEndDocument();
        if (xml.hasError()) {
            throw runtime_error("Unable to serialize preferences.");
        }
    }

    Preferences
    Preferences::deserialize(QIODevice& istrm)
    {
        Preferences rv;
        string text;

        /* Prepare to read XML from the data stream. */
        QXmlStreamReader xml(&istrm);

        /* The first token is not automatically loaded. */
        nextToken(xml,
                  false /* skipPureWhiteSpace */,
                  false /* skipComments */);

        /* Skip the "StartDocument" token. */
        expectToken(xml, QXmlStreamReader::StartDocument);
        nextToken(xml,
                  false /* skipPureWhiteSpace */,
                  false /* skipComments */);

        /* Expect start of root "harbinger" element. */
        expectToken(xml, QXmlStreamReader::StartElement, "harbinger");
        nextToken(xml);

        /* Expect the "advanceNotice" text element. */
        text = expectTextElement(xml, "advanceNotice");
        rv.advanceNotice = stringTo<unsigned long>(text);
        nextToken(xml);

        /* Expect the "screensaver" text element. */
        text = expectTextElement(xml, "screensaver");
        rv.screensaver = stringTo<unsigned long>(text);
        nextToken(xml);

        /* Expect the "message" text element. */
        text = expectTextElement(xml, "message");
        rv.message = text;
        nextToken(xml);

        /* Expect end of root "harbinger" element. */
        expectToken(xml, QXmlStreamReader::EndElement, "harbinger");
        nextToken(xml);

        /* Expect the "EndDocument" token. */
        expectToken(xml, QXmlStreamReader::EndDocument);

        return rv;
    }

}
