#ifndef PREFERENCES_DIALOG_H
#define PREFERENCES_DIALOG_H

#include "Preferences.h"
#include "ui_PreferencesDialog.h"
#include <sstream>
#include <QtCore/QtGlobal>
#include <QtWidgets/QtWidgets>

namespace Harbinger {

    /**
     * Dialog used to edit <c>Preferences</c> instances.
     */
    class PreferencesDialog : public QDialog,
                              public Ui::PreferencesDialog
    {
        Q_OBJECT

    public:

        /**
         * Construct a new <c>PreferencesDialog</c> instance having
         * parent <c>parent</c>.
         *
         * @param[in] parent parent
         */
        PreferencesDialog(QWidget* parent = NULL);

        /**
         * Destructor.
         */
        virtual
        ~PreferencesDialog();

        /**
         * Get the current set of preferences.  This method is
         * typically called before the dilaog is displayed to allow
         * the user to edit the preferences.
         *
         * @return current set of preferences
         */
        Preferences
        preferences() const;

        /**
         * Set the current set of preferences.  This method is
         * typically called after the dialog has been accepted to get
         * the changes made by the user.
         */
        void
        setPreferences(const Preferences& preferences);

    private:

        /**
         * Return the value of the <c>lineEdit</c> widget by
         * converting it from a string to an object of type <c>T</c>.
         *
         * @param[in] lineEdit <c>QLineEdit</c> widget
         * @return value
         */
        template<typename T> T
        getLineEdit(QLineEdit* lineEdit) const
        {
            T rv;
            std::istringstream istrm(qPrintable(lineEdit->text()));
            istrm >> rv;
            return rv;
        }

        /**
         * Set the text for the <c>lineEdit</c> widget by converting
         * the value <c>value</c> to a string.
         *
         * @param[in] lineEdit <c>QLineEdit</c> widget
         * @param[in] value value
         */
        template<typename T> void
        setLineEdit(QLineEdit* lineEdit,
                    const T& value)
        {
            std::ostringstream ostrm;
            ostrm << value;
            lineEdit->setText(QString::fromLatin1(ostrm.str().c_str()));
        }

    };

}

#endif  /*  PREFERENCES_DIALOG_H  */
